use crate::{get_pg, get_sqlite, get_sqlite_or_create};


pub enum DatabaseBackend {
    Sqlite,
    Postgres
}

pub struct Database {
    pub in_use: DatabaseBackend,
    pub sqlite: Option<sqlx::Pool<sqlx::Sqlite>>,
    pub postgres: Option<sqlx::Pool<sqlx::Postgres>>
}

impl Database {
    pub async fn new() -> Self {
        if std::env::var("DATABASE_URL").unwrap().contains("postgres") {
            return Self {
                in_use: DatabaseBackend::Postgres,
                sqlite: None,
                postgres: Some(get_pg!().clone())
            };
        } else {
            return Self {
                in_use: DatabaseBackend::Sqlite,
                sqlite: Some(get_sqlite_or_create!().clone()),
                postgres: None
            };
        }
    }
}

#[macro_export]
macro_rules! with {
    ($db:expr, $pg:block, $sqlite:block) => {{
        match $db.in_use {
            crate::db::DatabaseBackend::Postgres => {
                let pg = $db.postgres.clone().expect("Postgres connection not available");
                $pg
            }
            crate::db::DatabaseBackend::Sqlite => {
                let sqlite = $db.sqlite.clone().expect("Sqlite connection not available");
                $sqlite
            }
        }
    }};
}
